export declare const NUMBER_VECTORS: {
    type: string;
    value: bigint;
    hexadecimal: string;
}[];
export declare const DECODE_OUT_OF_RANGE_NUMBER_VECTORS: {
    type: string;
    hexadecimal: string;
}[];
export declare const OUT_OF_RANGE_NUMBER_VECTORS: {
    type: string;
    value: bigint;
}[];
export declare const INVALID_NUMBER_TYPE_VECTORS: string[];
