export declare const hardforks: {
    chainstart: any;
    homestead: any;
    dao: any;
    tangerineWhistle: any;
    spuriousDragon: any;
    byzantium: any;
    constantinople: any;
    petersburg: any;
    istanbul: any;
    muirGlacier: any;
    berlin: any;
    london: any;
    shanghai: any;
    arrowGlacier: any;
    grayGlacier: any;
    mergeForkIdTransition: any;
    merge: any;
    shardingFork: any;
};
//# sourceMappingURL=index.d.ts.map