/** EIP4844 constants */
export declare const MAX_CALLDATA_SIZE = 16777216;
export declare const MAX_ACCESS_LIST_SIZE = 16777216;
export declare const MAX_VERSIONED_HASHES_LIST_SIZE = 16777216;
export declare const LIMIT_BLOBS_PER_TX = 16777216;
export declare const MAX_TX_WRAP_KZG_COMMITMENTS = 16777216;
export declare const FIELD_ELEMENTS_PER_BLOB = 4096;
export declare const BYTES_PER_FIELD_ELEMENT = 32;
//# sourceMappingURL=constants.d.ts.map