/**
 *  @_ignore
 */
export declare function looseArrayify(hexString: string): Uint8Array;
export declare function zpad(value: String | number, length: number): String;
export declare function getPassword(password: string | Uint8Array): Uint8Array;
export declare function spelunk<T>(object: any, _path: string): T;
//# sourceMappingURL=utils.d.ts.map