"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wordlist = void 0;
var korean_1 = require("@scure/bip39/wordlists/korean");
Object.defineProperty(exports, "wordlist", { enumerable: true, get: function () { return korean_1.wordlist; } });
