import type { RlpStructuredDataish } from "./rlp.js";
/**
 *  Encodes %%object%% as an RLP-encoded [[DataHexString]].
 */
export declare function encodeRlp(object: RlpStructuredDataish): string;
//# sourceMappingURL=rlp-encode.d.ts.map