"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "ChecksumStruct", {
    enumerable: true,
    get: function() {
        return ChecksumStruct;
    }
});
const _superstruct = require("superstruct");
const _base64 = require("./base64");
const ChecksumStruct = (0, _superstruct.size)((0, _base64.base64)((0, _superstruct.string)(), {
    paddingRequired: true
}), 44, 44);

//# sourceMappingURL=checksum.js.map