import type { BytesLike } from "./index.js";
/**
 *  Returns the version 4 [[link-uuid]] for the %%randomBytes%%.
 *
 *  @see: https://www.ietf.org/rfc/rfc4122.txt (Section 4.4)
 */
export declare function uuidV4(randomBytes: BytesLike): string;
//# sourceMappingURL=uuid.d.ts.map