// Type definitions for tweetnacl-util

declare var util: util;
export = util;

interface util {
    decodeUTF8(s: string): Uint8Array;
    encodeUTF8(arr: Uint8Array): string;
    encodeBase64(arr: Uint8Array): string;
    decodeBase64(s: string): Uint8Array;
}
