import type { AccessList, AccessListish } from "./index.js";
/**
 *  Returns a [[AccessList]] from any ethers-supported access-list structure.
 */
export declare function accessListify(value: AccessListish): AccessList;
//# sourceMappingURL=accesslist.d.ts.map