"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    createProjectLogger: function() {
        return createProjectLogger;
    },
    createModuleLogger: function() {
        return createModuleLogger;
    }
});
const _debug = /*#__PURE__*/ _interop_require_default(require("debug"));
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const globalLogger = (0, _debug.default)('metamask');
function createProjectLogger(projectName) {
    return globalLogger.extend(projectName);
}
function createModuleLogger(projectLogger, moduleName) {
    return projectLogger.extend(moduleName);
}

//# sourceMappingURL=logging.js.map