/**
 * bn254 pairing-friendly curve.
 * Previously known as alt_bn_128, when it had 128-bit security.
 * Recent research shown it's weaker, the naming has been adjusted to its prime bit count.
 * https://github.com/zcash/zcash/issues/2502
 */
export declare const bn254: import("./abstract/weierstrass.js").CurveFn;
//# sourceMappingURL=bn254.d.ts.map