import type { BytesLike, RlpStructuredData } from "./index.js";
/**
 *  Decodes %%data%% into the structured data it represents.
 */
export declare function decodeRlp(_data: BytesLike): RlpStructuredData;
//# sourceMappingURL=rlp-decode.d.ts.map